#!/usr/bin/env python3

"""
This is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or
distribute this software, either in source code form or as a compiled
binary, for any purpose, commercial or non-commercial, and by any
means.

In jurisdictions that recognize copyright laws, the author or authors
of this software dedicate any and all copyright interest in the
software to the public domain. We make this dedication for the benefit
of the public at large and to the detriment of our heirs and
successors. We intend this dedication to be an overt act of
relinquishment in perpetuity of all present and future rights to this
software under copyright law.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

For more information, please refer to <http://unlicense.org/>
"""

from argparse import ArgumentParser, FileType
from hashlib import sha1
from struct import pack, unpack


def eulasetter(twlcfg, disagree):
	# Get config flags
	twlcfg.seek(0x88)
	flags, = unpack("<L", twlcfg.read(4))

	# Set EULA
	if disagree:
		flags &= ~(1 << 24)
	else:
		flags |= (1 << 24)

	# Write back to file
	twlcfg.seek(0x88)
	twlcfg.write(pack("<L", flags))

	# Update SHA1
	twlcfg.seek(0x88)
	sha = sha1(twlcfg.read(0x128)).digest()
	twlcfg.seek(0)
	twlcfg.write(sha)

	print("Successfully updated %s" % twlcfg.name)


if __name__ == "__main__":
	parser = ArgumentParser(description="Sets the EULA status of a DSi config file")
	parser.add_argument("twlcfg", metavar="TWLCFG#.dat", type=FileType("rb+"), nargs="*", help="TWLCFG files")
	parser.add_argument("-d", "--disagree", action="store_true", help="if set will disagree to EULA, default is agree")

	args = parser.parse_args()

	for twlcfg in args.twlcfg:
		eulasetter(twlcfg, args.disagree)
